'''
    pypsa2bb - create input data for the esm backbone, run backbone and analyse results
    Copyright (C) 2020-2022 Leonie Plaga, David Huckebrink, Christine Nowak, Jan Mutke, Jonas Finke, Silke Johanndeiter, Sophie Pathe

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import re
import yaml
from pathlib import Path
import shutil


def temporal_resolution(config):
    _model_fn = config["pypsa2bbPaths"]["pypsa_model_path"].split("/")[-1]
    _temporal_resolution = 0
    try:
        match = re.search(r"[^0-9]([0-9]+)H", _model_fn)
        _temporal_resolution = int(match.group(1))

    except IndexError:
        _temporal_resolution = 1
        print(
            "Warning: Unable to retrieve temporal resolution from filename, set to 1h."
        )

    return _temporal_resolution


def num_buses(config):
    _model_fn = config["pypsa2bbPaths"]["pypsa_model_path"].split("/")[-1]
    _num_lines = []

    # this might not be the finest method
    for ele in _model_fn.split("_"):
        try:
            _num_lines.append(int(ele))
        except Exception:
            pass

    if len(_num_lines) > 1:
        raise ValueError("Unable to retrieve number of buses from filename")

    return _num_lines[0]


scripts_dir = Path(__file__).parent
root_dir = scripts_dir.parent

normal_path = root_dir.joinpath("config.yaml")

if not root_dir.joinpath("config.yaml").exists():
    default = root_dir.joinpath("config.default.yaml").as_posix()
    shutil.copy(default, normal_path.as_posix())

config = yaml.safe_load(normal_path.open("r"))
