# %% [markdown]
# # Merge pypsa and hotmaps data
# 
# ## 1) find nuts3 regions that are within a pypsa-node
# Add a pypsa-node-name column for usage in pd.groupby

# %%

if "snakemake" not in locals():
    import snakemake
    snakemake.input ={
        "workbook" : "../results/elec_s_13_ec_lcopt_Co2L-24H.xlsx",
        "hotmaps_regional_demand_df" : "../data/Hotmaps_regional_demand.csv",
        "normalization_data" : "../data/hotmaps_task_2.7_load_profile_residential_shw_and_heating_yearlong_2010.csv",
        "heating_technologies" :"../data/Heating_technologies_EU28_v3.csv"
    }
    snakemake.output = ["../results/elec_s_13_ec_lcopt_Co2L-24H_heat.xlsx"]
    # }
    
    # snakemake = NestedNamespace(values)
    # setattr(snakemake,)
    snakemake.wildcards = {"opts":"Co2L-24H"}
    snakemake.params = {
        "nuts3_shapes_gdf": "../pypsa-eur/resources/nuts3_shapes.geojson",
        "pypsa_nodes_gdf": "../pypsa-eur/resources/regions_onshore_elec_s_13.geojson",
    }

# %%
import pandas as pd
import geopandas as gpd

pypsa_gdf = gpd.read_file(snakemake.params["pypsa_nodes_gdf"])

# todo: replace with snakemake.config["countries"]
countries = pypsa_gdf["name"].apply(lambda x: str(x)[:2]).unique()

# reduce nuts3_gdf and hotmaps_df to relevant countries
nuts3_gdf = gpd.read_file(snakemake.params["nuts3_shapes_gdf"])
nuts3_gdf = nuts3_gdf.query("country.isin(@countries)")

hotmaps_demand_df = pd.read_csv(snakemake.input["hotmaps_regional_demand_df"])
hotmaps_demand_df = hotmaps_demand_df.query("COUNTRY_CODE.isin(@countries)")

hotmaps_load_profiles_df = pd.read_csv(snakemake.input["normalization_data"])

# # project crs to be in meters (otherwise .centroid is incorrect):
nuts3_gdf.to_crs(3857, inplace=True)
pypsa_gdf.to_crs(3857, inplace=True)
# this did not lead to better results in finding the containing clusters

# %%
# # compute closest neighbouring cluster
pypsa_gdf["closest_cluster"] = ""
for i,row in pypsa_gdf.iterrows():
    cluster = row["geometry"]
    sub_frame = pypsa_gdf.loc[pypsa_gdf.name != row["name"],:]
    cluster_distances = sub_frame.distance(cluster.centroid)
    asc_sorted_distances = cluster_distances.sort_values().reset_index()
    # 0 would be `cluster` itself
    clostest_country_idx = asc_sorted_distances["index"][0]
    pypsa_gdf["closest_cluster"][i] = sub_frame["name"][clostest_country_idx]

pypsa_gdf

# %%

def get_containing_cluster(nuts_region):
    for i,cluster in enumerate(pypsa_gdf["geometry"]):
        if cluster.contains(nuts_region.centroid):
            return pypsa_gdf["name"][i]
    for i,cluster in enumerate(pypsa_gdf["geometry"]):
        if cluster.intersects(nuts_region):
                return pypsa_gdf["name"][i]


nuts3_gdf["cluster"] = nuts3_gdf["geometry"].apply(get_containing_cluster)


# TODO: there are some losses, that are insignificant for now
if any(pd.isna(nuts3_gdf["cluster"])):
    print("omitting the following nuts regions because no containing cluster was found\n:",
            nuts3_gdf.loc[pd.isna(nuts3_gdf["cluster"]),["index","pop","gdp", "country"]])
    nuts3_gdf = nuts3_gdf.loc[~pd.isna(nuts3_gdf["cluster"]),:]

cluster_matches_country = nuts3_gdf.country == nuts3_gdf.cluster.apply(lambda x: x[:2])
if not all(cluster_matches_country):
    print("omitting the following nuts regions because cluster does not match country was found\n:",
            nuts3_gdf.loc[~cluster_matches_country,["index","pop","gdp", "country","cluster"]])
    nuts3_gdf = nuts3_gdf.loc[cluster_matches_country,:]


# %%
nuts3_gdf.set_index("index",inplace=True)
nuts3_gdf

# %%
# hotmaps_demand_df.columns
# >> ['COUNTRY_CODE', 'NUTS_LEVEL', 'NUTS_CODE', 'NUTS_LABEL',
    #    'Residential_Buildings_2012', 'Buildings_Private_Service_Sector_2012',
    #    'Public_Buildings_2012', 'Total_SH_HW', 'Share_Total_SH_HW',
    #    'Share_Total_Residential_SH_HW', 'Share_Total_Non_res_service_SH_HW',
    #    'Share_Total_Non_res_public_SH_HW', 'District_Heat', 'Biomass',
    #    'Natural_Gas', 'Other_Renewables', 'Electricity', 'Other_Fossil',
    #    'District_Heat_Shares_NUTS', 'Biomass_Shares_NUTS',
    #    'Natural_Gas_Shares_NUTS', 'Other_RenewableShares_NUTS',
    #    'Electricity_Shares_NUTS', 'Other_Fossil_Shares_NUTS'],

hotmaps_demand_df.set_index("NUTS_CODE",inplace=True)

hotmaps_demand_df["res_heat_demand"] = hotmaps_demand_df["Total_SH_HW"] #* hotmaps_demand_df["Share_Total_Residential_SH_HW"]


# %%
# add heat hotmaps_demands to nuts3_gdf
demand_subframe = hotmaps_demand_df.loc[hotmaps_demand_df.index.isin(nuts3_gdf.index),:]
# convert from GWh to MWh
nuts3_gdf.loc[demand_subframe.index,"res_heat_demand"] = demand_subframe.loc[:,"res_heat_demand"] * 1000
nuts3_gdf


# %% [markdown]
# ## 2) select load profile based on frequency of nuts2 occurance

# %%
nuts3_gdf["nuts2"] = nuts3_gdf.index.to_series().apply(lambda x: x[:4])
for cluster in nuts3_gdf.cluster.unique():
    cluster_idcs = nuts3_gdf.cluster == cluster
    sub_frame = nuts3_gdf.loc[cluster_idcs,:]
    most_frequent = sub_frame["nuts2"].value_counts().index[0]
    nuts3_gdf.loc[cluster_idcs,"most_freq_nuts2"] = most_frequent

# %%
cluster_nuts_gdf = nuts3_gdf.groupby(["cluster","most_freq_nuts2","country"]).sum(numeric_only=True)
cluster_nuts_gdf = cluster_nuts_gdf.reset_index()
cluster_nuts_gdf["orig_cluster_name"] = cluster_nuts_gdf["cluster"]
cluster_nuts_gdf["cluster"] += " heat"
cluster_nuts_gdf 


# %%

# This is only for Norway, since there is no load profile data for Norway in Hotmaps
for i,most_freq_nut2 in enumerate(cluster_nuts_gdf.most_freq_nuts2):
    # this could also be reformulated as a function, because the closest_cluster 
    # is not necessarily present in NUTS2_code either...
    if most_freq_nut2 not in hotmaps_load_profiles_df["NUTS2_code"].unique():
        current_cluster = cluster_nuts_gdf["orig_cluster_name"][i]
        closest_cluster = pypsa_gdf.set_index("name")\
            .loc[current_cluster,"closest_cluster"]
        replacement_nuts2 = cluster_nuts_gdf.set_index("orig_cluster_name").loc[closest_cluster,"most_freq_nuts2"]
        print(f"Warning: replacing load profile (shape only) of '{most_freq_nut2}' with '{replacement_nuts2}'")
        cluster_nuts_gdf.loc[i,"most_freq_nuts2"] = replacement_nuts2


# %%
hotmaps_load_profiles_df = hotmaps_load_profiles_df.pivot(index="datetime",columns="NUTS2_code",values="load")

# %%
load_profile_sum = hotmaps_load_profiles_df[[*cluster_nuts_gdf.most_freq_nuts2]].sum()
normalized_profiles = hotmaps_load_profiles_df[[*cluster_nuts_gdf.most_freq_nuts2]].divide(load_profile_sum)

scaled_load_profiles = normalized_profiles.multiply(cluster_nuts_gdf["res_heat_demand"].values)
scaled_load_profiles

# %%
scaled_load_profiles.columns = cluster_nuts_gdf["cluster"]
scaled_load_profiles.columns.names = ["cluster"]
scaled_load_profiles.head()


# %%
scaled_load_profiles.index = pd.DatetimeIndex(scaled_load_profiles.index)

# %%
# get sampling freq from wildcards eg. from 'Co2L-24H'
sampling_freq = snakemake.wildcards["opts"].split("-")[-1]
resampled_demand_ts = scaled_load_profiles.resample(sampling_freq).sum()
resampled_demand_ts

# %%
resampled_demand_ts.sum()

# %% [markdown]
# ## Merge technology data

# %%

techs_path = snakemake.input["heating_technologies"]
heating_technologies = pd.read_csv(techs_path)


# %%

heating_technologies = heating_technologies.drop(['datetime', 'price_index_households_appliances', 'labour_cost_index_construction_sector', 'equipment_and_maintenance_index', 'equipment_cost_ share', 'installation_cost_share', 'k1_specific_investment_costs', 'k2_specific_investment_costs','InCost_CAGR', 'k1_fixed_O_and_M', 'k2_fixed_O_and_M', 'Unnamed: 21','Unnamed: 22', 'Unnamed: 23', 'Unnamed: 24', 'Unnamed: 25'], axis=1)
heating_technologies.head()

# %%
# to account for existing building stock, number of houses would have to be considered

# existing SFH --> ca. 10-15 kWth  -->  0,015 MWth 
# existing MFH --> max 400 kWth    -->  0,400 MWth 
# new SFH      --> ca. 4-10 kWth   -->  0,010 MWth 
# new MFH      --> max 160  kWth   -->  0,160 MWth 

# heating_technologies.loc[heating_technologies['type_of_building'] == 'existing SFH', 'installed_capacity'] = 15
# heating_technologies.loc[heating_technologies['type_of_building'] == 'existing MFH', 'installed_capacity'] = 400
# heating_technologies.loc[heating_technologies['type_of_building'] == 'new SFH', 'installed_capacity'] = 10
# heating_technologies.loc[heating_technologies['type_of_building'] == 'new MFH', 'installed_capacity'] = 160
# heating_technologies

# %%

r = snakemake.config["costs"]["fill_values"]["discount rate"]

heating_technologies['annuity'] = r/(1-(1+r)**(- heating_technologies['technical_lifetime']))

heating_technologies['annuity']

# %%
heating_technologies['specific_investment_costs']   *= 1000
heating_technologies['fixed_O_and_M']               *= 1000
heating_technologies['variable_O_and_M']            *= 1000
# heating_technologies['installed_capacity']          /= 1000
heating_technologies

# %%
new_heating_technologies = heating_technologies.loc[(heating_technologies['year'] == 2030),:] #& (heating_technologies['type_of_building'] == 'existing SFH')]
#new_heating_technologies

# %%

relevant_country_idx = new_heating_technologies['NUTS0_ID'].isin(countries)
techs_df = new_heating_technologies.loc[relevant_country_idx]
techs_df = techs_df.sort_values(by=['NUTS0_ID'])
techs_df = techs_df.reset_index(drop='index')
techs_df

# %%
# TODO: implement country specific building shares from https://gitlab.com/hotmaps/building-stock/-/blob/master/data/building_stock.csv
# while one could use 68 % SFH, 32 % MFH and (p32) and
# 22 % new, 78 % old (https://www.hotmaps-project.eu/wp-content/uploads/2018/03/D2.3-Hotmaps_for-upload_revised-final_.pdf) (p30)
# the weighting hardly affects the input data when compared to simple averages
#  note: there is no techno-economic data for non-residential buildings


# %% [markdown]
# ### aggregate heatpump and biomass technologies

# %%
def aggregated_name(name):
    if "bio" in name.lower() or "wood" in name.lower():
        return "Biomass boiler"
    elif "HP" in name:
        return "Heat pump"
    else:
        return name

techs_df["heating_equipment"] = techs_df["heating_equipment"].apply(aggregated_name)

# %%
agg_techs_df = techs_df.groupby(["country","NUTS0_ID","heating_equipment","year"]).mean(numeric_only=True)
agg_techs_df.reset_index(inplace=True)
agg_techs_df["country"] = agg_techs_df["NUTS0_ID"]
agg_techs_df.head()


# %%
# add mean tech params to countries that are not present in the techs_df
countries_w_techs = set(agg_techs_df.country.unique())
all_countries = set(cluster_nuts_gdf.country.unique())
countries_wo_techs = all_countries.difference(countries_w_techs)

mean_tech_params = agg_techs_df.groupby(["heating_equipment","year"]).mean(numeric_only=True).reset_index()


for country in countries_wo_techs:
    mean_tech_params["country"] = country
    mean_tech_params["NUTS0_ID"] = country
    agg_techs_df = pd.concat([agg_techs_df, mean_tech_params])



# %%
# merged_heating_technologies = pd.merge(heat_nodes_NUTS0_ID_heating_equipment_output, agg_techs_df)#
merged_heating_technologies = pd.merge(cluster_nuts_gdf, agg_techs_df)
merged_heating_technologies

# %% [markdown]
# ## Map units (techs) to nodes and grids

# %%
unit_input_grid_mapping = {
    "Natural gas": "fuel",
    "Oil boiler": "fuel",
    "Bio-oil boiler": "fuel",
    "Wood stove": "fuel",
    "Biomass_Manual": "fuel",
    "Biomass_Automatic": "fuel",
    # "DH substation": "district heating",
    # "Solar thermal":, 2b retrieved from influx
    "HP Brine-to-Water": "elec",
    "HP Air-to-Water": "elec",
    "Electric heater": "elec",
    "HP Air-to-Air": "elec",
    # newly aggregated units
    "Heat pump": "elec",
    "Biomass boiler": "fuel"
}

def get_local_node(row):
    output_node = unit_input_node_mapping.get(row["heating_equipment"],None)

    if output_node == None:
        if row["input_grid"] == "elec":
            # with "elec" being the default in the naming convention, convert name as follows
            # AT0 0 heat -> AT0 0
            return row["cluster"].replace(" heat","")
        else:
            print("no input node found for",row["heating_equipment"], ". Will be dropped.")
            return None
    else:
        return output_node
    

unit_input_node_mapping = {
    "Natural gas": "gas",
    "Oil boiler": "oil",
    "Wood stove": "biomass",
    "Biomass_Manual": "biomass",
    "Biomass_Automatic": "biomass",
    "Bio-oil boiler": "biomass",
    "Biomass boiler": "biomass",
    
    # The nodes below map to their clusters' electricity node
    # "HP Brine-to-Water": 
    # "HP Air-to-Water":
    # "Electric heater":
    # "HP Air-to-Air":
}

merged_heating_technologies["input_grid"] = merged_heating_technologies["heating_equipment"].apply(unit_input_grid_mapping.get)

merged_heating_technologies["input_node"] = merged_heating_technologies[["cluster","heating_equipment","input_grid"]].apply(get_local_node, axis=1)


merged_heating_technologies = merged_heating_technologies.loc[~merged_heating_technologies.input_node.isna(),:]
merged_heating_technologies.reset_index(inplace=True,drop=True)
    

# %%
merged_heating_technologies.country.unique()

# %%
merged_heating_technologies['Units'] = merged_heating_technologies['cluster'] + ' ' + merged_heating_technologies['heating_equipment']
merged_heating_technologies.head()

# %% [markdown]
# # Write data to Excel

# %%
from openpyxl import load_workbook

wb_path = snakemake.input["workbook"]
workbook = load_workbook(filename=wb_path)

grid_sheet = workbook['grid']
grid_sheet.append(['heat'])


# %%
node_sheet = workbook['node']

for item in cluster_nuts_gdf.cluster:
    node_sheet.append([item])


# %%
unit_sheet = workbook['unit']
for unit_name in merged_heating_technologies['Units']:
    unit_sheet.append([unit_name])

# %%
effLevelGroupUnit_sheet = workbook['effLevelGroupUnit']

for i in range(len(merged_heating_technologies)):
    effLevelGroupUnit_level1_list = ['level1' , 'directOff', merged_heating_technologies['Units'][i]]
    effLevelGroupUnit_sheet.append(effLevelGroupUnit_level1_list)

for i in range(len(merged_heating_technologies)):
    effLevelGroupUnit_level2_list = ['level2' , 'directOff', merged_heating_technologies['Units'][i]]
    effLevelGroupUnit_sheet.append(effLevelGroupUnit_level2_list)

for i in range(len(merged_heating_technologies)):
    effLevelGroupUnit_level3_list = ['level3' , 'directOff', merged_heating_technologies['Units'][i]]
    effLevelGroupUnit_sheet.append(effLevelGroupUnit_level3_list)


# %%
p_gn_sheet = workbook['p_gn']

# sheet headers are:
# grid	node	nodeBalance	   energyStoredPerUnitOfState
for node_name in cluster_nuts_gdf.cluster:
    new_row_p_gn = ["heat",node_name,1,0]
    p_gn_sheet.append(new_row_p_gn)


# %%
p_gnu_io_sheet = workbook['p_gnu_io']

# sheet headers are:
# grid	node	unit	input output	conversionCoeff	capacity	unitSize	invCosts	fomCosts	vomCosts	annuity	upperLimitCapacityRatio


for i in range(len(merged_heating_technologies)):
    new_row_p_gnu_output = ["heat", merged_heating_technologies['cluster'][i], merged_heating_technologies['Units'][i], 'output', 1, 'eps' , 1, merged_heating_technologies['specific_investment_costs'][i], merged_heating_technologies['fixed_O_and_M'][i], merged_heating_technologies['variable_O_and_M'][i], merged_heating_technologies['annuity'][i]]
    p_gnu_io_sheet.append(new_row_p_gnu_output)

for i in range(len(merged_heating_technologies)):
    new_row_p_gnu_input = [merged_heating_technologies['input_grid'][i], merged_heating_technologies['input_node'][i], merged_heating_technologies['Units'][i], 'input', 1,'', 1]
    p_gnu_io_sheet.append(new_row_p_gnu_input)



# %%
p_unit_sheet = workbook['p_unit']

# unit	eff00	availability	investMIP	minUnitCount	maxUnitCount


for i in range(len(merged_heating_technologies)):
    new_row_p_unit = [merged_heating_technologies['Units'][i], merged_heating_technologies["total_annual_net_efficiency"][i], 1, 0, 'eps', 'inf']
    #print(new_row_p_unit)
    p_unit_sheet.append(new_row_p_unit)

# %%
ts_influx_sheet = workbook["ts_influx"]
# grid	node	forecast index	t000000	t000001 t...n

for node_name in resampled_demand_ts:
        
    node_demand = -1*resampled_demand_ts[node_name]
    assert node_demand.dtype == float
    new_row_ts_influx = ['heat', node_name, 'f00', *node_demand]
    ts_influx_sheet.append(new_row_ts_influx)

# %%
utAvailabilityLimits_sheet = workbook['utAvailabilityLimits']

#unit	timesteps	becomeAvailable

for unit_name in merged_heating_technologies['Units']:
    #print(unit_name)
    new_row_utAvailabilityLimits = [unit_name, 't000001', 1]
    utAvailabilityLimits_sheet.append(new_row_utAvailabilityLimits)

# %% [markdown]
# # add emission limits

# %%
group_sheet = workbook["group"]
group_sheet.append(["emission group"])

# %%
# TODO: not all nodes in fuel grid have emissions! -> merge with p_nEmission
p_gn_vals = list(p_gn_sheet.values)

p_gn_df = pd.DataFrame(p_gn_vals[1:], columns=p_gn_vals[0])

gn_subframe = p_gn_df.query("grid == 'fuel'")[["grid","node"]]
gn_subframe["gngroup"] = "emission group"

gnGroup_sheet = workbook["gngroup"]
    
for i,row in gn_subframe.iterrows():
    gnGroup_sheet.append([*row])
    


# %%
sgroup_sheet = workbook["sgroup"]
sgroup_sheet.append(["s000", "emission group"])

# %%
p_groupPolicyEmission = workbook['p_groupPolicyEmission']
p_groupPolicyEmission.append(["emission group", "emissionCap", "CO2", "inf"])

# %% [markdown]
# # Save excel file

# %%
output_path = snakemake.output[0]
workbook.save(filename=output_path)
workbook.close()

# %%


# %%



