'''
    pypsa2bb - create input data for the esm backbone, run backbone and analyse results
    Copyright (C) 2020-2022 Leonie Plaga, David Huckebrink, Christine Nowak, Jan Mutke, Jonas Finke, Silke Johanndeiter, Sophie Pathe

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from matplotlib.colors import Normalize
import pandas as pd
from results import BackboneResult
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import numpy as np


#plots a map as background and pie charts for total generation
def plot_pieChart(inputPath, outputPath):
    #load Backbone results
    backbone_result = BackboneResult(inputPath)
    r_gnuTotalGenShare = backbone_result.param_as_df("r_gnuTotalGenShare")
    r_gnuTotalGenShare = r_gnuTotalGenShare.where(r_gnuTotalGenShare["value"]>0.01).where(r_gnuTotalGenShare["grid"]=="elec").dropna()

    #set size of pie charts to total generation of node
    maxPieSize = 0.25
    minPieSize = 0.08
    r_gnTotalGen = backbone_result.param_as_df("r_gnTotalGen")
    r_gnTotalGen = r_gnTotalGen.where(r_gnTotalGen["value"]>0.01).where(r_gnTotalGen["grid"]=="elec").dropna()
    minGen = min(r_gnTotalGen["value"])
    maxGen = max(r_gnTotalGen["value"])
    m = (maxPieSize-minPieSize)/(maxGen-minGen)
    b = maxPieSize-m*maxGen

    #must be updated manually according to used map: [x_position, y_position, unknown, size], one entry for each node
    country_position = [[.55, .76, .2, m*r_gnTotalGen["value"][0]+b], [.41, .705, .2, m*r_gnTotalGen["value"][1]+b], 
        [.075, .13, .2, m*r_gnTotalGen["value"][2]+b], [.237, .14, .2, m*r_gnTotalGen["value"][3]+b], 
        [.25, .485, .2, m*r_gnTotalGen["value"][4]+b], [.58, .27, .2, m*r_gnTotalGen["value"][5]+b], 
        [.42, .215, .2, m*r_gnTotalGen["value"][6]+b],[-.06, .125, .2, m*r_gnTotalGen["value"][7]+b]]

    #plot background map
    fig = plt.figure(figsize=(10,5.7),facecolor='white')
    ax = fig.add_axes([0, 0, 1, 1])
    ax.axis('off')
    im = plt.imread("F:/KlimawandelEinfluss/2021_08_VortragOR/node_structure.png")
    implot = plt.imshow(im)
    plt.xticks([])
    plt.yticks([])
    #plan legend, sorting here should be the same as numbers in "sorting"
    legend_list = list()
    legend_list.append(mpatches.Patch(color='lightblue', label='hydro dam'))
    legend_list.append(mpatches.Patch(color='royalblue', label='run of river'))
    legend_list.append(mpatches.Patch(color='midnightblue', label='pumped hydro'))
    legend_list.append(mpatches.Patch(color='yellow', label='PV'))
    legend_list.append(mpatches.Patch(color='green', label='wind'))
    legend_list.append(mpatches.Patch(color='sienna', label='biomass'))
    legend_list.append(mpatches.Patch(color='lavender', label='geothermal'))
    legend_list.append(mpatches.Patch(color='orange', label='nuclear'))
    legend_list.append(mpatches.Patch(color='black', label='coal'))
    legend_list.append(mpatches.Patch(color='grey', label='gas'))
    plt.legend(handles=legend_list, loc=7)

    #prepare plot data
    node_list = r_gnuTotalGenShare["node"].drop_duplicates()
    for ix,element in enumerate(node_list):
        a = plt.axes(country_position[ix], facecolor='w')
        country_df = r_gnuTotalGenShare.where(r_gnuTotalGenShare["node"]==element).dropna().drop(columns=["grid", "node"]).reset_index()
        country_df['sorting'] = 0
        country_df['color'] = "empty"
        #combine unit(types) with colors and sort them
        for ix,element in enumerate(country_df["unit"]):
            if "coal" in element or "lignite" in element:
                country_df.loc[ix, "color",]="black"
                country_df.loc[ix, "sorting"] = 8
            elif "solar" in element:
                country_df.loc[ix, "color",]="yellow"
                country_df.loc[ix, "sorting"] = 3
            elif "wind" in element:
                country_df.loc[ix, "color",] = "green"
                country_df.loc[ix, "sorting"] = 4
            elif "hydro" in element:
                country_df.loc[ix, "color",] = "lightblue"
                country_df.loc[ix, "sorting"] = 0
            elif "ror" in element:
                country_df.loc[ix, "color",] = "royalblue"
                country_df.loc[ix, "sorting"] = 1
            elif "PHS" in element:
                country_df.loc[ix, "color",] = "midnightblue"
                country_df.loc[ix, "sorting"] = 2
            elif "CCGT" in element or "OCGT" in element:
                country_df.loc[ix, "color",] = "grey"
                country_df.loc[ix, "sorting"] = 9
            elif "nuclear" in element:
                country_df.loc[ix, "color",] = "orange"
                country_df.loc[ix, "sorting"] = 7
            elif "biomass" in element:
                country_df.loc[ix, "color",] = "sienna"
                country_df.loc[ix, "sorting"] = 5
            elif "geothermal" in element:
                country_df.loc[ix, "color"] = "lavender"
                country_df.loc[ix, "sorting"] = 6
        country_df=country_df.sort_values("sorting")
        a.pie(country_df["value"], startangle=90, counterclock=False, normalize=True, colors=country_df["color"])
        a.axis('equal')  # Equal aspect ratio ensures that pie is drawn as a circle.


    plt.savefig(outputPath, dpi=300)

