import subprocess as sp
import sys

sys.path.append(".")

from scripts.helpers import excel_to_gdx
from pathlib import Path
import os


def test_input_equality():
    # to be able to run this test invoke
    # snakemake -j8 --keep-target-files results/elec_s_5_ec_lcopt_Co2L-24H.xlsx
    # with only germany (as in this config: https://github.com/PyPSA/pypsa-eur/blob/b88322587f4caf2b66a1ccd5a2a4d3eef2b948bc/test/config.test1.yaml)

    xlsx_base_file = "results/elec_s_5_ec_lcopt_Co2L-24H.xlsx"

    target_gdx_fn = "tests/assets/elec_s_5_ec_lcopt_Co2L-24H.py.gdx"
    existing_gdx_fn = "tests/assets/elec_s_5_ec_lcopt_Co2L-24H.gdxxrw.gdx"

    # create the gdx file
    excel_to_gdx(xlsx_base_file, target_gdx_fn)

    # compare the created gdx with the previously created gdx
    completed_process = sp.run(["gdxdiff", target_gdx_fn,
                               existing_gdx_fn, "eps=0.0000000001"], stdout=sp.PIPE)
    
    stdout_text = completed_process.stdout.decode()
    output_line = stdout_text.splitlines()[-2]
    diff_file = output_line.split(" ")[-1]
    
    diff_dump = sp.run(["gdxdump", diff_file],stdout=sp.PIPE).stdout.decode()

    assert "No differences" in stdout_text, \
        f"Differences were found!\n{diff_dump}"

    # clean up
    os.remove(target_gdx_fn)
    os.remove(diff_file)
    
